
; Test Program to test the 58167 Interrupt line with the 8259A	
; on the PIC/RTC S-100 board.            by John Monahan
;
;	V1.0	6/3/2010		;First version
;	V1.6	7/7/2010
;
PICportA	EQU	020H		;8259A ports	
PICportB	EQU	021H
RTCSEL		EQU	0A0H		;58167 ports
RTCDATA		EQU	0A1H		

;
;SD Systems Video Board for character I/O
;
CONOUT$PORT	EQU	1H
CONIN$PORT	EQU	1H
CONSTAT$PORT	EQU	0H	

CR	EQU	0DH
LF	EQU	0AH
ESC	EQU	1BH

	ORG	100H

begin:	LXI	SP,STACK
	LXI     H,SIGN$ON		;print a welcome message
	CALL	PSTRING
	CALL	ZCI			;Wait for start key
	LXI     H,CRLF$MSG		;print a CR/LF
	CALL	PSTRING
	
	
	DI				;No INT's until we get things setup
;	DB	0EDH, 5EH		;IM2 mode for Z80 (Uses [I] register for high byte address)
;	DB	0EDH, 56H		;IM1 mode for Z80 (Jump to 38H in RAM always)
	DB	0EDH, 46H		;IM0 8080 mode
;	MVI	A,0H			;Set [I] register for 0H
;	DB	0EDH,47H		;Mov I,A
	
					;First we need to setup the RST vectors in low memory					
					;Setup Z80 Int. Mode 0 Int. Jump vectors (0 is used by CP/M, skip)
	MVI	A,0C3H			;@08H JMP RST1 Vector Location
	STA	0008H
	LXI	H,RST1
	SHLD	009H
	
	STA	0010H			;@10H JMP to RST2 vector location
	LXI	H,RST2
	SHLD	11H
	
	STA	0018H			;@18H JMP to RST3 vector location
	LXI	H,RST3
	SHLD	19H

	STA	0020H			;@20H JMP to RST4 vector location
	LXI	H,RST4
	SHLD	21H

	STA	0028H			;@28H JMP to RST5 vector location
	LXI	H,RST5
	SHLD	29H

	STA	0030H			;@30H JMP to RST6 vector location
	LXI	H,RST6
	SHLD	31H

	STA	0038H			;@38H JMP to RST7 vector location
	LXI	H,RST7
	SHLD	39H


					;Setup the 8259A PIC Controller (for Poll mode)
					;NOTE It is very important that the 8259A is LEVEL 
					;triggered because the the Z80 IE is off from time to time
					;and may miss an interrupt. The clock chip would then 
					;not be reset for the next interrupt.
					
	MVI	A,00011111B		;Set for A5-7 of service routine=0
					;LEVEL triggered, Routine interval=4
					;Single 8259, and IVW4 needed
	OUT	PICportA		;Send it (This is ICW1)
	
	MVI	A,00H			;Upper Byte of service routine table (00H) Does not really matter
					;since we are running in poll mode here anyway
	OUT	PICportB		;Send it (ICW2)
						
	OUT	PICportB		;No slaves ICW3
	
	MVI	A,00000010B		;Set not fully nested, non buffered mode, NON-auto EOI 
					;and not 8086 mode
	OUT	PICportB		;Send it (ICW4)
	
					;Now send Output control bytes....
	
	MVI	A,0H			;Enable ALL bits
	OUT	PICportB		;Send it (0CW1)
	
	MVI	A,10100000B		;Set to auto rotating priority
	OUT	PICportA		;Send it (OCW2)
			
	MVI	A,00001100B		;Set 8259A to Poll mode 
	OUT	PICportA		;Send it, (OCW3)	
;

	MVI	A,11H			;Point to the 58167A Clock chip Interrupt Control Register
	OUT	RTCSEL	
	MVI	A,00000100B		;>>>> Set to 1 Second interrupt <<<<
	OUT	RTCDATA			;With this pin #13 of chip will go low after 1 second

	EI				;Int's can now be handled in poll mode
	IN	PICportA		;It's not clear to me why but one input here is required
					;to get the ints/8259A going

LOOP:	EI
	NOP				;Allow a slice of time for ints to get through
	NOP
	NOP
	NOP
	DI
	MVI	C,'.'
	CALL	ZCO
	CALL	ZCSTS			;Anything at Keyboard?
	JZ	LOOP
	CALL	ZCI
	CPI	ESC			;Abort back to CPM (0H, Reboot) if an ESC key
	JNZ	LOOP
	JMP	0H
	
;
;-----------------------  Interrupt routines ----------------------------------
;					
RST1:	DI
	PUSH	H
	LXI     H,SIGN$MODE0		;print a welcome message
	JMP	IntRoutine
	
RST2:	DI				;Arrive here from  RST2 (10H in RAM)
	PUSH	PSW
	PUSH	B
	PUSH	D			
	PUSH	H
					;Need to get Int status right away
	MVI	A,00001111B		;Send 8259A a Poll command, Reads IS Register on next IN PicPortA
	OUT	PICportA		;Send it, (OCW3)
	IN	PICportA		;Get and show Bit pattern returned.
	STA	IntStatus		;Store it for later display

	MVI	A,10H			;Point to Clock chip Interrupt Status Register
	OUT	RTCSEL
	IN	RTCDATA			;Note this read will reset the chip interrupt for next time
	STA	ClockStatus

	LXI	H,SIGN$MODE1
	CALL	PSTRING
	
	LDA	IntStatus		;What were the status bits
	CALL	ZBITS			;Display the 8259A poll status				

	LXI	H,CLK$MSG
	CALL	PSTRING

	LDA	ClockStatus		;What were the status bits
	CALL	ZBITS			;Display the 8259A poll status				
	CALL	ZCRLF

	MVI	A,20H			;Send EOI now
	OUT	PICportA		;Send it (OCW2)
	
	POP	H
	POP	D
	POP	B
	POP	PSW
	RET
	
RST3:	DI
	PUSH	H
	LXI     H,SIGN$MODE2		
	JMP	IntRoutine

RST4:	DI
	PUSH	H
	LXI     H,SIGN$MODE3		
	JMP	IntRoutine

RST5:	DI
	PUSH	H
	LXI     H,SIGN$MODE4		
	JMP	IntRoutine
	
RST6:	DI
	PUSH	H
	LXI     H,SIGN$MODE5		
	JMP	IntRoutine

RST7:	DI
	PUSH	H
	LXI     H,SIGN$MODE6		
	JMP	IntRoutine


IntRoutine:
	PUSH	PSW
	PUSH	B
					;Need to get Int status right away
	MVI	A,00001111B		;Send 8259A a Poll command, Reads IS Register on next IN PicPortA
	OUT	PICportA		;Send it, (OCW3)
	IN	PICportA		;Get and show Bit pattern returned.
	STA	IntStatus		;Store it for later display
	CALL	PSTRING			;print what HL point to
	LDA	IntStatus		;What were the status bits
	CALL	ZBITS			;Display the 8259A poll status				

	MVI	A,20H			;Send EOI now
	OUT	PICportA		;Send it (OCW2)
	POP	B
	POP	PSW
	POP	H
	RET
	
	
;--------------------- SUPPORT ROUTINES ----------------------------------

ZCO:	PUSH	PSW		;Write character that is in [C]
ZCO1:	IN   	0H		;Show Character
	ANI	04H
	JZ	ZCO1
	MOV	A,C
	OUT	CONOUT$PORT
	POP	PSW
	RET

ZCSTS:	IN	0H		;Get Keyboard Status
	ANI	02H
	RET

ZCI:	IN	CONSTAT$PORT	;Return keyboard character in [A]
	ANI	02H
	JZ	ZCI
	IN	CONIN$PORT
	RET

PSTRING:MOV	A,M		;Print string @ [HL] up to 0
	ORA	A
	RZ	
	MOV	C,A
	CALL	ZCO	
	INX	H
	JMP	PSTRING

PHEX:	
	PUSH	PSW		;Print HEX [A]
	PUSH	B
	PUSH	PSW
	RRC
	RRC
	RRC
	RRC
	CALL	ZCONV
	POP	PSW
	CALL	ZCONV
	MVI	C,' '
	CALL	ZCO
	POP	B
	POP	PSW
	RET
;
ZCONV:	
	ANI	0FH		;HEX to ASCII and print it
	ADI	90H
	DAA
	ACI	40H
	DAA
	MOV	C,A
	CALL	ZCO
	RET
;
ZCRLF:
	PUSH	PSW
	MVI	C,CR
	CALL	ZCO
	MVI	C,LF
	CALL	ZCO
	POP	PSW
	RET
;
;DISPLAY BIT PATTERN IN [A]
;
ZBITS:	PUSH	PSW
	PUSH	B
	PUSH	D
	MOV	E,A		
	MVI	B,8
BQ2:	DB	0CBH,23H	;Z80 Op code for SLA A,E	
	MVI	A,18H
	ADC	A
	MOV	C,A
	CALL	ZCO
	DCR	B
	JNZ	BQ2
	MVI	C,' '
	CALL	ZCO
	POP	D
	POP	B
	POP	PSW
	CALL	ZCRLF
	RET
;
SIGN$ON:	DB	'PIC/RTC board 1 Second Interrupt Test Program (V1.6)',CR,LF
		DB	'Will use Z80 interrupt mode IM2 [I=0].',CR,LF
		DB	'Connect the 58167 Interrupt line (P39) to one of',CR,LF
		DB	'the V0-7 lines (P37) AND connect the same Vector line input',CR,LF
		DB	'to U13 (P36).',CR,LF
		DB	'Note also, the 1 second interrupt routine assumes a RST2',CR,LF
		DB	'8080 type interrupt and so will be located at 10H in RAM.',CR,LF
		DB	'(SW2 = o,o,c,o, c,o,o,o)',CR,LF
		DB	'Press any key to start. Esc to Abort...',0
CRLF$MSG:	DB	CR,LF,0
SIGN$MODE0:	DB	CR,LF,'Jump to Int Routine at 08H,'
		DB	' 8259A Poll status bits = ',0
SIGN$MODE1:	DB	CR,LF,'Jump to Int Routine at 10H,'
		DB	' 8259A Poll status bits = ',0
SIGN$MODE2:	DB	CR,LF,'Jump to Int Routine at 18H,'
		DB	' 8259A Poll status bits = ',0
SIGN$MODE3:	DB	CR,LF,'Jump to Int Routine at 20H,'
		DB	' 8259A Poll status bits = ',0
SIGN$MODE4:	DB	CR,LF,'Jump to Int Routine at 28H,'
		DB	' 8259A Poll status bits = ',0
SIGN$MODE5:	DB	CR,LF,'Jump to Int Routine at 30H,'
		DB	' 8259A Poll status bits = ',0
SIGN$MODE6:	DB	CR,LF,'Jump to Int Routine at 38H,'
		DB	' 8259A Poll status bits = ',0
CLK$MSG:	DB	'MM58167A Interrupt Reg = ',0

IntStatus:	DB	0H			;Store 8259A poll status bits here
ClockStatus:	DB	0H

	DS	200H			;Make a large stack since we have non stop ints
STACK:	DW	0H
STACKSTORE DW	0H
